<?php

/**
 * WP_Meteor
 *
 * @package   WP_Meteor
 * @author    Aleksandr Guidrevitch <alex@excitingstartup.com>
 * @copyright 2020 wp-meteor.com
 * @license   GPL 2.0+
 * @link      https://wp-meteor.com
 */

namespace WP_Meteor\Blocker\FirstInteraction;

/**
 * Provide Import and Export of the settings of the plugin
 */
class UltimateReorder extends Base
{
    public $adminPriority = -1;
    public $priority = 99;
    public $tab = 'ultimate';
    public $title = 'Maximum available speed';
    public $description = ""; //"Delays script loading to 2 seconds";
    public $disabledInUltimateMode = false;
    public $defaultEnabled = false;

    public $pattern = [['.*', '']];

    public function backend_display_settings()
    {
        echo '<div id="' . $this->id . '" class="ultimate"
                    data-prefix="' . $this->id . '" 
                    data-title="' . $this->title . '"></div>';
    }

    /* triggered from wpmeteor_load_settings */
    public function load_settings($settings)
    {
        $settings[$this->id] = isset($settings[$this->id])
            ? $settings[$this->id]
            : ['enabled' => $this->defaultEnabled];

        $settings[$this->id]['id'] = $this->id;
        // $settings[$this->id]['regexp'] = $this->pattern;
        $settings[$this->id]['after'] = 'REORDER';
        $settings[$this->id]['description'] = $this->description;
        return $settings;
    }

    public function frontend_rewrite($buffer, $settings)
    {
        return preg_replace_callback('/<script[^>]*?>/i', function ($matches) {
            list($tag) = $matches;

            $result = $tag;
            if (!preg_match('/\s+data-src=/i', $tag) && !preg_match('/data-wpmeteor-bypass="true"/', $tag)) {
                if (preg_match('/\s+src=/i', $tag)) {
                    $result = preg_replace('/\s+src=/', " data-wpmeteor-id=\"{$this->id}\" data-wpmeteor-after=\"REORDER\" data-src=", $tag);
                } else {
                    if (preg_match('/\s+type=/i', $tag)) {
                        $result = preg_replace('/\s+type=([\'"])text\/javascript\1/', " data-wpmeteor-id=\"{$this->id}\" data-wpmeteor-after=\"REORDER\" type=\"javascript/blocked\"", $tag);
                    } else {
                        $result = preg_replace('/<script/', "<script data-wpmeteor-id=\"{$this->id}\" data-wpmeteor-after=\"REORDER\" type=\"javascript/blocked\"", $tag);
                    }
                }
            }
            return preg_replace('/\s*data-wpmeteor-bypass="true"/i', '', $result);
        }, $buffer);
    }

    public function frontend_adjust_wpmeteor($wpmeteor, $settings)
    {
        if ($settings[$this->id]['enabled']) {
            $wpmeteor['rdelay'] = 2000;
        } else {
            $wpmeteor['rdelay'] = 1000;
        }
        return $wpmeteor;
    }
}
