/**
 *   WP Meteor Wordpress Plugin
 *   Copyright (C) 2020  Aleksandr Guidrevitch
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
import jQueryMock from './includes/mocks/jquery';
import dispatcher from './includes/utils/dispatcher';
import listenerOptions from './includes/utils/listener-options';
import delta from './includes/utils/delta';

const DCL = 'DOMContentLoaded';
const separator = "----";
const S = 'SCRIPT';
const I = 'requestIdleCallback';
const N = null;
const c = process.env.DEBUG ? console.log : () => {};

(function (w, d, a, r, L, E, m, k, t, n) {

    const reorder = [];

    let DOMContentLoaded = false;
    let DOMContentLoadedQueue = [];
    let WindowLoaded = false;
    let WindowLoadedQueue = [];
    let emulatedDOMContentLoadedFired = false;
    let emulatedWindowLoadedFired = false;

    let iterating = true;

    if ('NodeList' in w && !NodeList.prototype.forEach) {
        NodeList.prototype.forEach = function (callback, thisArg) {
            thisArg = thisArg || w;
            for (var i = 0; i < this.length; i++) {
                callback.call(thisArg, this[i], i, this);
            }
        };
    }

    const nextTick = setTimeout;

    if (process.env.DEBUG) {
        d[a](DCL, () => {
            c(delta(), separator, DCL);
        });

        dispatcher.on('l', () => {
            c(delta(), separator, "L");
        });

        w[a](L, () => {
            c(delta(), separator, L);
        });
    }

    // saving original methods
    const dOrigAddEventListener = d[a].bind(d);
    const dOrigRemoveEventListener = d[r].bind(d);
    const wOrigAddEventListener = w[a].bind(w);
    const wOrigRemoveEventListener = w[r].bind(w);
    const dOrigCreateElement = d.createElement.bind(d);

    const emulatedWindowLoaded = () => {
        iterating = false;
        dispatcher.emit('l');
        emulatedDOMContentLoadedFired = true;

        let func;
        while (func = WindowLoadedQueue.shift()) {
            try {
                func.call(N, WindowLoaded);
            } catch (e) {
                console.error(e);
            }
        }
        emulatedWindowLoadedFired = true;
    }

    dOrigAddEventListener(DCL, (e) => DOMContentLoaded = e);
    wOrigAddEventListener(L, (e) => WindowLoaded = e); // L = load

    let i = 0;
    const iterate = () => {
        process.env.DEBUG && c(delta(), 'it', i++, reorder.length);
        const element = reorder.shift();
        if (element) {
            // process.env.DEBUG && c(separator, "iterating", element, element.dataset);
            // const ds = element.dataset;
            if (element.dataset.src) {
                // process.env.DEBUG && c(delta(), "src", element);
                if (element.hasAttribute('async')) {
                    // process.env.DEBUG && c(delta(), "src", element);
                    unblock(element);
                    nextTick(iterate);
                } else {
                    unblock(element, iterate, iterate);
                }
            } else if (element.type == 'javascript/blocked') {
                unblock(element);
                // allow inserted script to execute
                nextTick(iterate);
            } else if (element.loaded === 0) {
                process.env.DEBUG && c(delta(), 'still waiting for', element.src)
                element[a](L, iterate); // L = load
                element[a](E, iterate); // E = error
            } else {
                // it might be wrongfully processed script by backend, eg type="application/ld+json" 
                // and execution will stop here
                // process.env.DEBUG && c("running next iteration");
                nextTick(iterate);
            }
        } else {
            // process.env.DEBUG && c('loaded all the scripts');
            // not restoring original addEventListener
            // to avoid unexpected failures, 
            // however, that triggers spurious handlers which were sleeping
            // d[a] = dOrigAddEventListener;
            if (DOMContentLoadedQueue.length) {
                // process.env.DEBUG && c('running DOMContentLoadedQueue');
                let func;
                while (func = DOMContentLoadedQueue.shift()) {
                    try {
                        func.call(N, DOMContentLoaded);
                    } catch (e) {
                        console.error(e);
                    }
                }
                // in case new script tags (blocked) were added from DOMContentLoaded handlers
                // they should have been captured by MutationObserver
                nextTick(iterate);
            } else if (WindowLoaded) {
                // process.env.DEBUG && c('running emulatedWindowLoaded');
                emulatedWindowLoaded();
            } else {
                wOrigAddEventListener(L, emulatedWindowLoaded); // L = load
            }
        }
    };

    const unblock = (el, onLoad, onError) => {
        const ds = el.dataset;
        if (ds.src) {
            process.env.DEBUG && c(delta(), "unblocked", ds.src);
            el.bypass = true;
            if (onLoad)
                el[a](L, () => nextTick(onLoad)); // L = load
            if (onError)
                el[a](E, () => nextTick(onError)); // E = error
            el.src = ds.src;
            delete ds.src;
        } else if (el.type === 'javascript/blocked') {
            // onLoad is never passed here
            process.env.DEBUG && c(delta(), "unblocked inline", el);
            const newElement = d.createElement(S);
            const attrs = el.attributes;
            for (var i = attrs.length - 1; i >= 0; i--) {
                newElement.setAttribute(attrs[i].name, attrs[i].value);
            }
            newElement.text = el.text;
            newElement.bypass = true;
            newElement.type = 'text/javascript';
            // move attributes

            const parentNode = el.parentNode;
            parentNode.removeChild(el);
            parentNode.appendChild(newElement);
        } else {
            process.env.DEBUG && c(delta(), "already unblocked", el.src);
        }
    }

    // Capturing and queueing DOMContentLoaded event handlers
    d[a] = (event, func, ...args) => {
        if (event === DCL) {
            if (!emulatedDOMContentLoadedFired) {
                DOMContentLoadedQueue.push(func);
            } else {
                process.env.DEBUG && c(delta(), "late " + DCL);
                func(DOMContentLoaded);
            }
            return void (0);
        }
        return dOrigAddEventListener(event, func, ...args);
    }

    d[r] = (event, func) => {
        if (!emulatedDOMContentLoadedFired && event === DCL) {
            DOMContentLoadedQueue = DOMContentLoadedQueue.filter(f => f !== func);
            return void (0);
        }
        return dOrigRemoveEventListener(event, func);
    };

    const preload = () => reorder.forEach(script => {
        const src = script.dataset.src;
        if (src) {
            var s = dOrigCreateElement('link');
            s.rel = 'pre' + L;
            s.as = 'script';
            s.href = src;
            s.crossorigin = true;
            d.head.appendChild(s);
            process.env.DEBUG && c(delta(), 'preloading', src);
        }
    });

    if (location.href.match(/wpmeteordisable/) || !w.MutationObserver) {
        dOrigAddEventListener(DCL, () => {
            d.querySelectorAll('script[data-wpmeteor-after]').forEach(el => reorder.push(el));
            preload();
            iterate();
        });
        return;
    }

    new jQueryMock();

    d.createElement = function (...args) {
        // If this is not a script tag, bypass
        // dont rely on window loaded or document loaded as the tags might 
        // be inserted long after this
        if (args[0].toUpperCase() !== S) {

            // Binding to document is essential
            return dOrigCreateElement(...args)
        }

        const scriptElt = dOrigCreateElement(...args);
        // scriptElt.blackListed = false;

        // Backup the original setAttribute function
        const originalSetAttribute = scriptElt.setAttribute.bind(scriptElt)
        const originalGetter = scriptElt.__proto__.__lookupGetter__('src').bind(scriptElt);

        Object.defineProperties(scriptElt, {
            'src': {
                get: originalGetter,
                set(value) {
                    if (scriptElt.bypass) {
                        // process.env.DEBUG && c('bypass for', value.toString());
                        return originalSetAttribute('src', value);
                    }
                    originalSetAttribute('data-wpmeteor-after', 'REORDER');
                    return scriptElt.dataset.src = value;
                }
            }
        })

        // Monkey patch the setAttribute function so that the setter is called instead.
        // Otherwise, setAttribute('type', 'whatever') will bypass our custom descriptors!
        scriptElt.setAttribute = function (name, value) {
            if (name === 'src')
                scriptElt[name] = value
            else
                HTMLScriptElement.prototype.setAttribute.call(scriptElt, name, value)
        }

        return scriptElt
    }

    // have to find scripts before us
    const observer = new MutationObserver(mutations => {
        mutations.forEach(({ addedNodes }) => {
            addedNodes.forEach(node => {
                // For each added script tag
                if (node.nodeType === 1) {
                    if (S === node.tagName && !node.bypass) {
                        const ds = node.dataset;
                        // const hasReorder = ds.wpmeteorAfter === 'REORDER';
                        // process.env.DEBUG && c([ds.src, node.src, ds.src && !node.src, node.dataset]);
                        // if (ds.src && !node.src || hasReorder) {
                        if (ds.wpmeteorAfter === 'REORDER') {
                            // process.env.DEBUG && c('pushing', node)
                            process.env.DEBUG && c(delta(), "blocked", node.tagName, ds.src || node);
                            reorder.push(node);
                            if (!iterating) {
                                // we have to restart iterate() to insert missing scripts
                                process.env.DEBUG && c(delta(), 'restarting reordering');
                                iterating = true;
                                nextTick(iterate);
                            }
                        } else if (node.src || node.getAttribute("src")) {
                            process.env.DEBUG && c(delta(), "detected", node.getAttribute("src"));
                            node.loaded = 0;
                            node[a](L, () => node.loaded = 1); // L = loaded
                            node[a](E, () => node.loaded = 1); // E = error
                            reorder.push(node);
                        } else {
                            // console.error('missed', node);
                        }
                    }
                }
            })
        })
    });

    /* we have to override document.write as all of them will fire after DOMContentLoaded */
    d.write = (...args) => {
        if (d.currentScript) { // that implicitely means DOMContentLoad already fired
            d.currentScript.insertAdjacentHTML('afterend', ...args);
        } else {
            console.error("Unable to document.write");
            // dOrigWrite.bind(d)(...args);
        }
    }

    // Capturing and queueing Window Load event handlers
    w[a] = (event, func, ...args) => {
        if (event === L) { // L = load
            if (!emulatedWindowLoadedFired) {
                WindowLoadedQueue.push(func);
            } else {
                process.env.DEBUG && c(delta(), "late Window load");
                func(WindowLoaded);
            }
            return;
        }
        // process.env.DEBUG && c(event, func);
        return wOrigAddEventListener(event, func, ...args);
    }

    w[r] = (event, func) => {
        if (!emulatedWindowLoadedFired && event === L) { // L = load
            WindowLoadedQueue = WindowLoadedQueue.filter(f => f !== func);
            return;
        }
        return wOrigRemoveEventListener(event, func);
    };

    dispatcher.on('l', () => {
        observer.disconnect();
        d.createElement = dOrigCreateElement;
    })

    let firstInteractionFired = false;
    const onFirstInteraction = () => {
        const b = d.body; // at the moment of initialization document.body is not present
        b[r](m, onFirstInteraction, listenerOptions); // mouseover
        b[r](k, onFirstInteraction, listenerOptions); // keydown
        b[r](t, onFirstInteraction, listenerOptions); // touchstart
        b[r](n, onFirstInteraction, listenerOptions); // wheel
        wOrigRemoveEventListener(L, onFirstInteraction); // L = load

        if (!firstInteractionFired) {
            firstInteractionFired = true;
            preload();
            nextTick(iterate);
        }
    }

    dOrigAddEventListener(DCL, () => {
        const b = d.body; // at the moment of initialization document.body is not present
        b[a](m, onFirstInteraction, listenerOptions); // mouseover
        b[a](k, onFirstInteraction, listenerOptions); // keydown
        b[a](t, onFirstInteraction, listenerOptions); // touchstart
        b[a](n, onFirstInteraction, listenerOptions); // wheel
    });

    const callback = setTimeout.bind(N, onFirstInteraction, _wpmeteor.rdelay);
    wOrigAddEventListener(L, () => {
        if (I in w) {
            process.env.DEBUG && w[I](() => c(delta(), separator, I));
            w[I](callback);
        } else {
            callback();
        }
    }); // L = load

    // Starts the monitoring
    observer.observe(d.documentElement, {
        attributes: true,
        childList: true,
        subtree: true
    });

})(window,
    document,
    'addEventListener',
    'removeEventListener',
    'load',
    'error',
    'mouseover',
    'keydown',
    'touchstart',
    'wheel');
